package com.vcarecity.cvs.service.impl;

import com.google.inject.Inject;
import com.google.inject.assistedinject.Assisted;
import com.vcarecity.cvs.core.ColumnUpdateMapper;
import com.vcarecity.cvs.properties.AppProperties;
import com.vcarecity.cvs.service.ResultHandlerService;
import com.vcarecity.cvs.service.TimerService;
import com.vcarecity.cvs.util.CreateDatasource;
import com.zaxxer.hikari.HikariDataSource;
import lombok.extern.slf4j.Slf4j;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.util.Arrays;
import java.util.List;
import java.util.concurrent.atomic.AtomicLong;

/**
 * @author Kerry on 19/11/21
 */


@Slf4j
public class MySQLResultHandlerServiceImpl implements ResultHandlerService, TimerService {

    private final HikariDataSource dataSource;

    private AtomicLong writeTime = new AtomicLong(0L);

    @Inject
    public MySQLResultHandlerServiceImpl(AppProperties properties, @Assisted Object sql) {
        this.dataSource = CreateDatasource.createDataSource(properties.getToDb());
    }


    @Override
    public <T> void resultHandler(String table, String[] header, List<T> dataList) throws Exception {

        final String sql = getSql(table, header);

        long startTime = System.currentTimeMillis();

        logger.debug("{},{}", dataList.size(), sql);
        final Connection connection = dataSource.getConnection();
        try {
            connection.setAutoCommit(false);
            PreparedStatement ps = connection.prepareStatement(sql);

            int index = 0;
            for (T t : dataList) {
                index++;
                if (t.getClass().isArray()) {
                    Object[] rows = (Object[]) t;
                    for (int i = 0; i < rows.length; i++) {
                        ps.setObject(i + 1, rows[i]);
                    }
                } else if (List.class.isAssignableFrom(t.getClass())) {
                    List rows = (List) t;
                    for (int i = 0; i < rows.size(); i++) {
                        ps.setObject(i + 1, rows.get(i));
                    }
                }
                ps.addBatch();
                //if (index % 1000 == 0) {
                //    ps.executeBatch();
                //    connection.commit();
                //}
            }
            ps.executeBatch();
            connection.commit();
            ps.close();
        } finally {
            connection.close();
        }

        final long curUseTime = System.currentTimeMillis() - startTime;

        writeTime.addAndGet(curUseTime);

        logger.info("WRITE SUCCESS to mysql usage time {} /ms, size = {}", curUseTime, dataList.size());
    }


    private static String getSql(String table, String[] headers) {

        final String[] newHeaders = ColumnUpdateMapper.updateColumnName(table, headers);
        final String column = String.join(",", newHeaders);

        String[] fill = new String[headers.length];
        Arrays.fill(fill, "?");
        final String join = String.join(",", fill);

        return "INSERT INTO " + table + "(" + column + ") VALUES(" + join + ")";

    }

    @Override
    public long getUsageTime() {
        return this.writeTime.longValue();
    }
}
